"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addBabelPlugin = addBabelPlugin;
exports.addBabelPreset = addBabelPreset;
exports.babelConfigurationHooks = void 0;

var _plugins = require("@sewing-kit/plugins");

const PLUGIN = 'SewingKit.Babel';
const babelConfigurationHooks = (0, _plugins.createProjectPlugin)(PLUGIN, ({
  project,
  tasks: {
    build,
    test,
    dev
  }
}) => {
  const addHooks = hooks => ({ ...hooks,
    babelConfig: new _plugins.WaterfallHook(),
    babelIgnorePatterns: new _plugins.WaterfallHook(),
    babelExtensions: project instanceof _plugins.Package ? new _plugins.WaterfallHook() : undefined
  });

  build.hook(({
    hooks
  }) => {
    hooks.configureHooks.hook(addHooks);
  });
  dev.hook(({
    hooks
  }) => {
    hooks.configureHooks.hook(addHooks);
  });
  test.hook(({
    hooks
  }) => {
    hooks.configureHooks.hook(addHooks);
  });
});
exports.babelConfigurationHooks = babelConfigurationHooks;

function addBabelPlugin(plugin) {
  const id = `${PLUGIN}.AddBabelPlugin`;
  return createProjectPluginTargettingAllConfigurationHooks(id, hooks => {
    var _hooks$babelConfig;

    (_hooks$babelConfig = hooks.babelConfig) === null || _hooks$babelConfig === void 0 ? void 0 : _hooks$babelConfig.hook(config => {
      var _config$plugins;

      return { ...config,
        plugins: [...((_config$plugins = config.plugins) !== null && _config$plugins !== void 0 ? _config$plugins : []), plugin]
      };
    });
  });
}

function addBabelPreset(preset) {
  const id = `${PLUGIN}.AddBabelPreset`;
  return createProjectPluginTargettingAllConfigurationHooks(id, hooks => {
    var _hooks$babelConfig2;

    (_hooks$babelConfig2 = hooks.babelConfig) === null || _hooks$babelConfig2 === void 0 ? void 0 : _hooks$babelConfig2.hook(config => {
      var _config$presets;

      return { ...config,
        presets: [...((_config$presets = config.presets) !== null && _config$presets !== void 0 ? _config$presets : []), preset]
      };
    });
  });
}

function createProjectPluginTargettingAllConfigurationHooks(id, configurator) {
  return (0, _plugins.createProjectPlugin)(id, ({
    tasks: {
      build,
      dev,
      test
    }
  }) => {
    build.hook(({
      hooks
    }) => {
      hooks.configure.hook(configurator);
    });
    dev.hook(({
      hooks
    }) => {
      hooks.configure.hook(configurator);
    });
    test.hook(({
      hooks
    }) => {
      hooks.configure.hook(configurator);
    });
  });
}