"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createCompileBabelStep = createCompileBabelStep;

var _path = require("path");

var _fsExtra = require("fs-extra");

var _glob = require("glob");

var _plugins = require("@sewing-kit/plugins");

function createCompileBabelStep({
  project: pkg,
  api
}, config, options) {
  return api.createStep({
    id: 'Babel.Compile',
    label: 'compile with babel'
  }, async step => {
    const {
      configFile = 'babel.js',
      outputPath,
      extension
    } = options;
    const babelConfigPath = api.configPath(`build/packages/${pkg.name}/${configFile}`);

    if (config.babelConfig == null || config.babelIgnorePatterns == null || config.babelExtensions == null) {
      throw new _plugins.MissingPluginError('@sewing-kit/plugin-babel');
    }

    const [babelConfig, babelIgnorePatterns] = await Promise.all([config.babelConfig.run({}), config.babelIgnorePatterns.run([])]);
    await api.write(babelConfigPath, `module.exports=${JSON.stringify(babelConfig)};`);
    const extensions = await config.babelExtensions.run([]);
    const sourceRoot = (0, _path.resolve)(pkg.root, 'src');
    await step.exec('node_modules/.bin/babel', [sourceRoot, ...(0, _plugins.toArgs)({
      outDir: outputPath,
      configFile: babelConfigPath,
      verbose: true,
      noBabelrc: true,
      babelConfig: false,
      extensions: extensions.join(','),
      ignore: babelIgnorePatterns.length > 0 ? babelIgnorePatterns.join(',') : undefined
    }, {
      dasherize: true
    })]);

    if (extension) {
      const replaceJsWith = extension.startsWith('.') ? extension : `.${extension}`;
      await Promise.all((0, _glob.sync)('**/*.js', {
        cwd: outputPath,
        absolute: true
      }).map(file => (0, _fsExtra.rename)(file, file.replace(/\.js$/, replaceJsWith))));
    }
  });
}