"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs_extra_1 = require("fs-extra");
const glob_1 = require("glob");
const plugins_1 = require("@sewing-kit/plugins");
function createCompileBabelStep({ project: pkg, api }, config, options) {
    return api.createStep({ id: 'Babel.Compile', label: 'compile with babel' }, async (step) => {
        const { configFile = 'babel.js', outputPath, extension } = options;
        const babelConfigPath = api.configPath(`build/packages/${pkg.name}/${configFile}`);
        if (config.babelConfig == null ||
            config.babelIgnorePatterns == null ||
            config.babelExtensions == null) {
            throw new plugins_1.MissingPluginError('@sewing-kit/plugin-babel');
        }
        const [babelConfig, babelIgnorePatterns] = await Promise.all([
            config.babelConfig.run({}),
            config.babelIgnorePatterns.run([]),
        ]);
        await api.write(babelConfigPath, `module.exports=${JSON.stringify(babelConfig)};`);
        const extensions = await config.babelExtensions.run([]);
        const sourceRoot = path_1.resolve(pkg.root, 'src');
        await step.exec('node_modules/.bin/babel', [
            sourceRoot,
            ...plugins_1.toArgs({
                outDir: outputPath,
                configFile: babelConfigPath,
                verbose: true,
                noBabelrc: true,
                babelConfig: false,
                extensions: extensions.join(','),
                ignore: babelIgnorePatterns.length > 0
                    ? babelIgnorePatterns.join(',')
                    : undefined,
            }, { dasherize: true }),
        ]);
        if (extension) {
            const replaceJsWith = extension.startsWith('.')
                ? extension
                : `.${extension}`;
            await Promise.all(glob_1.sync('**/*.js', { cwd: outputPath, absolute: true }).map((file) => fs_extra_1.rename(file, file.replace(/\.js$/, replaceJsWith))));
        }
    });
}
exports.createCompileBabelStep = createCompileBabelStep;
